﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.MetadirectoryServices;
using System.IO;
using System.Collections.ObjectModel;
using System.Security;

namespace FIM.PowerShell
{
    public class MAExtensible2
        : FIMPowerShell,
          IMAExtensible2GetCapabilities,
          IMAExtensible2GetHierarchy,
          IMAExtensible2GetSchema,
          IMAExtensible2GetParameters,
          IMAExtensible2GetPartitions,
          IMAExtensible2Password,
          IMAExtensible2CallImport,
          IMAExtensible2CallExport
    {
        public MAExtensible2()
        {
            // Initialize manually since the API won't call Initialize/Terminate
            this.Initialize();
        }

        #region IMAExtensible2GetCapabilities
        public MACapabilities Capabilities
        {
            get 
            {
                string scriptPath = FIMUtils.FindMaScript("Get-MACapabilities.ps1");

                if (!File.Exists(scriptPath))
                {
                    throw new EntryPointNotImplementedException();
                }

                var obj = this.Invoke(this.CreateRuntime(scriptPath))
                            .FirstOrDefault();

                if (obj != null)
                {
                    if (obj.BaseObject is MACapabilities)
                    {
                        return (MACapabilities)obj.BaseObject;
                    }
                }

                throw new EntryPointNotImplementedException("The returned object for Get-MACapabilities.ps1 is not an instance of MACapabilities.");
            }
        }
        #endregion

        #region IMAExtensible2GetHierarchy
        public IList<HierarchyNode> GetHierarchy(KeyedCollection<string, ConfigParameter> configParameters, HierarchyNode parent)
        {
            string scriptPath = FIMUtils.FindMaScript("Get-MAHierarchy.ps1");

            if (!File.Exists(scriptPath))
            {
                throw new EntryPointNotImplementedException();
            }

            var runtime = this.CreateRuntime(scriptPath)
                            .AddParameter("ConfigParameters", configParameters)
                            .AddParameter("Parent", parent);

            return this.Invoke(runtime)
                        .Select(i => i.BaseObject)
                        .OfType<HierarchyNode>()
                        .ToList();
        }
        #endregion

        #region IMAExtensible2GetSchema
        public Schema GetSchema(KeyedCollection<string, ConfigParameter> configParameters)
        {
            string scriptPath = FIMUtils.FindMaScript("Get-MASchema.ps1");

            if (!File.Exists(scriptPath))
            {
                throw new EntryPointNotImplementedException();
            }

            var obj = this.Invoke(this.CreateRuntime(scriptPath).AddParameter("ConfigParameters", configParameters))
                        .FirstOrDefault();

            if (obj != null)
            {
                if (obj.BaseObject is Schema)
                {
                    return (Schema)obj.BaseObject;
                }
            }

            throw new EntryPointNotImplementedException("The returned object for Get-MASchema.ps1 is not an instance of Schema.");
        }
        #endregion

        #region IMAExtensible2GetParameters
        public IList<ConfigParameterDefinition> GetConfigParameters(KeyedCollection<string, ConfigParameter> configParameters, ConfigParameterPage page)
        {
            string scriptPath = FIMUtils.FindMaScript("Get-MAConfigParameters.ps1");

            if (!File.Exists(scriptPath))
            {
                throw new EntryPointNotImplementedException();
            }

            var runtime = this.CreateRuntime(scriptPath)
                            .AddParameter("ConfigParameters", configParameters)
                            .AddParameter("Page", page);

            return this.Invoke(runtime)
                        .Select(i => i.BaseObject)
                        .OfType<ConfigParameterDefinition>()
                        .ToList();
        }

        public ParameterValidationResult ValidateConfigParameters(KeyedCollection<string, ConfigParameter> configParameters, ConfigParameterPage page)
        {
            string scriptPath = FIMUtils.FindMaScript("Test-MAConfigParameters.ps1");

            if (!File.Exists(scriptPath))
            {
                throw new EntryPointNotImplementedException();
            }

            var runtime = this.CreateRuntime(scriptPath)
                            .AddParameter("ConfigParameters", configParameters)
                            .AddParameter("Page", page);

            var obj = this.Invoke(runtime).FirstOrDefault();

            if (obj != null)
            {
                if (obj.BaseObject is ParameterValidationResult)
                {
                    return (ParameterValidationResult)obj.BaseObject;
                }
            }

            throw new EntryPointNotImplementedException("The returned object for Test-MAConfigParameters.ps1 is not an instance of ParameterValidationResult.");
        }
        #endregion

        #region IMAExtensible2GetPartitions
        public IList<Partition> GetPartitions(KeyedCollection<string, ConfigParameter> configParameters)
        {
            string scriptPath = FIMUtils.FindMaScript("Get-MAPartitions.ps1");

            if (!File.Exists(scriptPath))
            {
                throw new EntryPointNotImplementedException();
            }

            var runtime = this.CreateRuntime(scriptPath)
                            .AddParameter("ConfigParameters", configParameters); ;

            return this.Invoke(runtime)
                        .Select(i => i.BaseObject)
                        .OfType<Partition>()
                        .ToList();
        }
        #endregion

        #region IMAExtensible2Password
        public void ChangePassword(CSEntry csentry, SecureString oldPassword, SecureString newPassword)
        {
            string scriptPath = FIMUtils.FindMaScript("Invoke-MAChangePassword.ps1");

            if (!File.Exists(scriptPath))
            {
                throw new EntryPointNotImplementedException();
            }

            var runtime = this.CreateRuntime(scriptPath)
                            .AddParameter("CSEntry", csentry)
                            .AddParameter("OldPassword", oldPassword)
                            .AddParameter("NewPassowrd", newPassword);

            this.Invoke(runtime);
        }

        public void ClosePasswordConnection()
        {
            string scriptPath = FIMUtils.FindMaScript("Invoke-MAClosePasswordConnection.ps1");

            if (!File.Exists(scriptPath))
            {
                throw new EntryPointNotImplementedException();
            }

            var runtime = this.CreateRuntime(scriptPath);

            this.Invoke(runtime);
        }

        public ConnectionSecurityLevel GetConnectionSecurityLevel()
        {
            string scriptPath = FIMUtils.FindMaScript("Get-MAConnectionSecurityLevel.ps1");

            if (!File.Exists(scriptPath))
            {
                throw new EntryPointNotImplementedException();
            }

            var runtime = this.CreateRuntime(scriptPath);

            var obj = this.Invoke(runtime).FirstOrDefault();

            if (obj != null)
            {
                if (obj.BaseObject is ConnectionSecurityLevel)
                {
                    return (ConnectionSecurityLevel)obj.BaseObject;
                }
            }

            throw new EntryPointNotImplementedException("The returned object for Get-MAConnectionSecurityLevel.ps1 is not an instance of ConnectionSecurityLevel.");
        }

        public void OpenPasswordConnection(KeyedCollection<string, ConfigParameter> configParameters, Partition partition)
        {
            string scriptPath = FIMUtils.FindMaScript("Invoke-MAOpenPasswordConnection.ps1");

            if (!File.Exists(scriptPath))
            {
                throw new EntryPointNotImplementedException();
            }

            var runtime = this.CreateRuntime(scriptPath)
                            .AddParameter("ConfigParameters", configParameters)
                            .AddParameter("Partition", partition);

            this.Invoke(runtime);
        }

        public void SetPassword(CSEntry csentry, SecureString newPassword, PasswordOptions options)
        {
            string scriptPath = FIMUtils.FindMaScript("Invoke-MASetPassword.ps1");

            if (!File.Exists(scriptPath))
            {
                throw new EntryPointNotImplementedException();
            }

            var runtime = this.CreateRuntime(scriptPath)
                            .AddParameter("CSEntry", csentry)
                            .AddParameter("NewPassowrd", newPassword)
                            .AddParameter("Options", options);

            this.Invoke(runtime);
        }
        #endregion

        #region IMAExtensible2CallImport
        public CloseImportConnectionResults CloseImportConnection(CloseImportConnectionRunStep importRunStep)
        {
            string scriptPath = FIMUtils.FindMaScript("Invoke-MACloseImportConnection.ps1");

            if (!File.Exists(scriptPath))
            {
                throw new EntryPointNotImplementedException();
            }

            var runtime = this.CreateRuntime(scriptPath)
                            .AddParameter("ImportRunStep", importRunStep);

            var obj = this.Invoke(runtime).FirstOrDefault();

            if (obj != null)
            {
                if (obj.BaseObject is CloseImportConnectionResults)
                {
                    return (CloseImportConnectionResults)obj.BaseObject;
                }
            }

            throw new EntryPointNotImplementedException("The returned object for Invoke-MACloseImportConnection.ps1 is not an instance of CloseImportConnectionResults.");
        }

        public GetImportEntriesResults GetImportEntries(GetImportEntriesRunStep importRunStep)
        {
            string scriptPath = FIMUtils.FindMaScript("Get-MAImportEntries.ps1");

            if (!File.Exists(scriptPath))
            {
                throw new EntryPointNotImplementedException();
            }

            var runtime = this.CreateRuntime(scriptPath)
                            .AddParameter("ImportRunStep", importRunStep);

            var obj = this.Invoke(runtime).FirstOrDefault();

            if (obj != null)
            {
                if (obj.BaseObject is GetImportEntriesResults)
                {
                    return (GetImportEntriesResults)obj.BaseObject;
                }
            }

            throw new EntryPointNotImplementedException("The returned object for Get-MAImportEntries.ps1 is not an instance of GetImportEntriesResults.");
        }

        public int ImportDefaultPageSize
        {
            get 
            {
                string scriptPath = FIMUtils.FindMaScript("Get-MAImportDefaultPageSize.ps1");

                if (!File.Exists(scriptPath))
                {
                    throw new EntryPointNotImplementedException();
                }

                var runtime = this.CreateRuntime(scriptPath);

                var obj = this.Invoke(runtime).FirstOrDefault();

                if (obj != null)
                {
                    if (obj.BaseObject is int)
                    {
                        return (int)obj.BaseObject;
                    }
                }

                throw new EntryPointNotImplementedException("The returned object for Get-MAImportDefaultPageSize.ps1 is not an Int32 value.");
            }
        }

        public int ImportMaxPageSize
        {
            get
            {
                string scriptPath = FIMUtils.FindMaScript("Get-MAImportMaxPageSize.ps1");

                if (!File.Exists(scriptPath))
                {
                    throw new EntryPointNotImplementedException();
                }

                var runtime = this.CreateRuntime(scriptPath);

                var obj = this.Invoke(runtime).FirstOrDefault();

                if (obj != null)
                {
                    if (obj.BaseObject is int)
                    {
                        return (int)obj.BaseObject;
                    }
                }

                throw new EntryPointNotImplementedException("The returned object for Get-MAImportMaxPageSize.ps1 is not an Int32 value.");
            }
        }

        public OpenImportConnectionResults OpenImportConnection(KeyedCollection<string, ConfigParameter> configParameters, Schema types, OpenImportConnectionRunStep importRunStep)
        {
            string scriptPath = FIMUtils.FindMaScript("Invoke-MAOpenImportConnection.ps1");

            if (!File.Exists(scriptPath))
            {
                throw new EntryPointNotImplementedException();
            }

            var runtime = this.CreateRuntime(scriptPath)
                            .AddParameter("ConfigParameters", configParameters)
                            .AddParameter("Types", types)
                            .AddParameter("ImportRunStep", importRunStep);

            var obj = this.Invoke(runtime).FirstOrDefault();

            if (obj != null)
            {
                if (obj.BaseObject is OpenImportConnectionResults)
                {
                    return (OpenImportConnectionResults)obj.BaseObject;
                }
            }

            throw new EntryPointNotImplementedException("The returned object for Invoke-MAOpenImportConnection.ps1 is not an instance of OpenImportConnectionResults.");
        }
        #endregion

        #region IMAExtensible2CallExport
        public void CloseExportConnection(CloseExportConnectionRunStep exportRunStep)
        {
            string scriptPath = FIMUtils.FindMaScript("Invoke-MACloseExportConnection.ps1");

            if (!File.Exists(scriptPath))
            {
                throw new EntryPointNotImplementedException();
            }

            var runtime = this.CreateRuntime(scriptPath)
                            .AddParameter("ExportRunStep", exportRunStep);

            this.Invoke(runtime);
        }

        public int ExportDefaultPageSize
        {
            get
            {
                string scriptPath = FIMUtils.FindMaScript("Get-MAExportDefaultPageSize.ps1");

                if (!File.Exists(scriptPath))
                {
                    throw new EntryPointNotImplementedException();
                }

                var runtime = this.CreateRuntime(scriptPath);

                var obj = this.Invoke(runtime).FirstOrDefault();

                if (obj != null)
                {
                    if (obj.BaseObject is int)
                    {
                        return (int)obj.BaseObject;
                    }
                }

                throw new EntryPointNotImplementedException("The returned object for Get-MAExportDefaultPageSize.ps1 is not an Int32 value.");
            }
        }

        public int ExportMaxPageSize
        {
            get
            {
                string scriptPath = FIMUtils.FindMaScript("Get-MAExportMaxPageSize.ps1");

                if (!File.Exists(scriptPath))
                {
                    throw new EntryPointNotImplementedException();
                }

                var runtime = this.CreateRuntime(scriptPath);

                var obj = this.Invoke(runtime).FirstOrDefault();

                if (obj != null)
                {
                    if (obj.BaseObject is int)
                    {
                        return (int)obj.BaseObject;
                    }
                }

                throw new EntryPointNotImplementedException("The returned object for Get-MAExportMaxPageSize.ps1 is not an Int32 value.");
            }
        }

        public void OpenExportConnection(KeyedCollection<string, ConfigParameter> configParameters, Schema types, OpenExportConnectionRunStep exportRunStep)
        {
            string scriptPath = FIMUtils.FindMaScript("Invoke-MAOpenExportConnection.ps1");

            if (!File.Exists(scriptPath))
            {
                throw new EntryPointNotImplementedException();
            }

            var runtime = this.CreateRuntime(scriptPath)
                            .AddParameter("ConfigParameters", configParameters)
                            .AddParameter("Types", types)
                            .AddParameter("ExportRunStep", exportRunStep);

            this.Invoke(runtime);
        }

        public PutExportEntriesResults PutExportEntries(IList<CSEntryChange> csentries)
        {
            string scriptPath = FIMUtils.FindMaScript("Invoke-MAPutExportEntries.ps1");

            if (!File.Exists(scriptPath))
            {
                throw new EntryPointNotImplementedException();
            }

            var runtime = this.CreateRuntime(scriptPath)
                            .AddParameter("CSEntries", csentries);

            var obj = this.Invoke(runtime).FirstOrDefault();

            if (obj != null)
            {
                if (obj.BaseObject is PutExportEntriesResults)
                {
                    return (PutExportEntriesResults)obj.BaseObject;
                }
            }

            throw new EntryPointNotImplementedException("The returned object for Invoke-MAPutExportEntries.ps1 is not an instance of PutExportEntriesResults.");
        }
        #endregion
    }
}
